/*
// Copyright 1995-1998 - Just For Fun Software, Inc., all rights reserved.
// Author:  George Woltman
// Email: woltman@magicnet.net
//
*/

#include "prime.h"
#include <math.h>
#define TRUE 1
#define FALSE 0

/* Each FFT word is multiplied by a two-to-phi value.  These */
/* routines set and get the FFT value without the two-to-phi */
/* multiplier. */

void get_fft_value (
	unsigned long p,
	unsigned long fftlen,
	unsigned long i,
	long	*retval)
{
	double	*dblp, tmp, val, pow;
	unsigned long b, s;

/* Get address of fft value */

	dblp = (double *) addr (p, fftlen, i);

/* The first FFT word is not multiplied by anything. */

	if (i == 0) {
		*retval = (long) *dblp;
		return;
	}

/* Compute the number of big FFT words. */

	b = p % fftlen;

/* Compute the number of small FFT words. */

	s = fftlen - b;

/* Divide by two-to-phi to generate an integer. */
/* Do not trust the pow routine.  The assembly code will */
/* generate the value with greater accuracy. */

	val = *dblp;
	pow = (double) mulmod (s, i, fftlen) / (double) fftlen;
	__asm__ __volatile__ (
		"fldl %1\n fldl %2\n f2xm1\n fld1\n faddp\n fdivrp\n frndint\n fstpl %0" 
		: "=m" (tmp)
		: "m" (val), "m" (pow)
	);
	*retval = (long) tmp;
}

void set_fft_value (
	unsigned long p,
	unsigned long fftlen,
	unsigned long i,
	long	val)
{
	double	*dblp, tmp, pow;
	unsigned long b, s;

/* Get address of fft value */

	dblp = (double *) addr (p, fftlen, i);

/* The first FFT word is not multiplied by anything. */

	if (i == 0) {
		*dblp = val;
		return;
	}

/* Compute the number of big FFT words. */

	b = p % fftlen;

/* Compute the number of small FFT words. */

	s = fftlen - b;

/* Multiply value by two-to-phi before setting fft value. */
/* Do not trust the pow routine.  The assembly code will */
/* generate the value with greater accuracy. */

	pow = (double) mulmod (s, i, fftlen) / (double) fftlen;
	__asm__ __volatile__ (
		"fldl %1\n f2xm1\n fld1\n faddp\n fimull %2\n fstpl %0"
		: "=m" (tmp)
		: "m" (pow), "m" (val)
	);
	*dblp = tmp;
}
