/* Copyright 1995-1998 Just For Fun Software, Inc. */
/* Author:  George Woltman */
/* Email: woltman@magicnet.net */

/* Include files */

#include "prime.h"
#include <string.h>

/* Routine definitions */

void rangeStatus ();
void options_cpu ();

/* Get line from the user (stdin) */

void get_line (
	char	*buf)
{
	char	c;
	int	i;
	for (i = 0; ; i++) {
		if (_read (0, &c, 1) != 1) break;
		if (c == '\n' || c == 0) break;
		if (i < 80) *buf++ = c;
	}
	*buf++ = 0;
}

/* Get a number from the user */

unsigned long get_number (
	unsigned long dflt)
{
	char	line[80];
	unsigned long i;
	get_line (line);
	if (line[0] == 0) return (dflt);
	return (atol (line));
}

/* Ask a Yes/No question */

void askYN (
	char	*str,
	int	*val)
{
	char	buf[80];
	printf ("%s (%s): ", str, *val ? "Y" : "N");
	get_line (buf);
	if (buf[0] == 0) return;
	*val = (buf[0] == 'Y' || buf[0] == 'y');
}

/* Ask a number question */

void askNum (
	char	*str,
	unsigned long *val,
	unsigned long min,
	unsigned long max)
{
	char	buf[80];
	unsigned long newval;
	printf ("%s (%ld): ", str, *val);
loop:	get_line (buf);
	if (buf[0] == 0) return;
	newval = atol (buf);
	if (min || max) {
		if (newval < min || newval > max) {
			printf ("Please enter a value between %ld and %ld. ",
				min, max);
			goto loop;
		}
	}
	*val = newval;
}

/* Ask a number question */

void askNumNoDflt (
	char	*str,
	unsigned long *val,
	unsigned long min,
	unsigned long max)
{
	char	buf[80];
	unsigned long newval;
	printf ("%s: ", str);
loop:	get_line (buf);
	if (buf[0] == 0) goto loop;
	newval = atol (buf);
	if (min || max) {
		if (newval < min || newval > max) {
			printf ("Please enter a value between %ld and %ld. ",
				min, max);
			goto loop;
		}
	}
	*val = newval;
}

/* Ask a string question */

void askStr (
	char	*str,
	char	*val,
	unsigned long maxlen)
{
	char	buf[80];
	if (val[0])
		printf ("%s (%s): ", str, val);
	else
		printf ("%s: ", str);
loop:	get_line (buf);
	if (buf[0] == 0) return;
	if (strlen (buf) > maxlen) {
		printf ("Maximum string length is %ld characters. ", maxlen);
		goto loop;
	}
	strcpy (val, buf);
}

/* Wait for user input - gives the user time to read the screen */

void askOK ()
{
	char	str[80];
	if (THREAD_KILL) return;
	printf ("\nHit enter to continue: ");
	get_line (str);
}

/* Ask user if he is satisfied with his dialog responses */

int askOkCancel ()
{
	char	buf[80];
	if (THREAD_KILL) return (FALSE);
	printf ("\nAccept the answers above? (Y): ");
	get_line (buf);
	return (buf[0] == 0 || buf[0] == 'Y' || buf[0] == 'y');
}

/* Output a long string with a max of 75 characters to a line */

void outputLongLine (
	char	*buf)
{
	char	line[80];
	char	*p;
	int	i, j;

	for (p = buf; ; ) {
		for (i = 0; i < 75; i++) {
			line[i] = p[i];
			if (p[i] == 0 || p[i] == '\n') { j = i; break; }
			if (p[i] == ' ' || p[i] == '.' || p[i] == ',') j = i;
		}
		line[j+1] = 0;
		printf (line);
		if (p[j] != '\n') printf ("\n");
		if (p[j] == 0) break;
		p += j + 1;
		while (*p == ' ') p++;
	}
}

/* Test/Primenet dialog */

void test_primenet ()
{
	int	m_primenet, m_dialup, m_work_dflt;
	int	m_lucas, m_factor, m_dblchk, m_new_dates;
	unsigned long m_work;
	short	work_pref;

	m_primenet = USE_PRIMENET;
	m_dialup = DIAL_UP;
	m_work = DAYS_OF_WORK;
	if (WORK_PREFERENCE == 0) {
		m_work_dflt = 1;
		work_pref = default_work_type ();
	} else {
		m_work_dflt = 0;
		work_pref = WORK_PREFERENCE;
	}
	m_lucas = !! (work_pref & PRIMENET_ASSIGN_TEST);
	m_factor = !! (work_pref & PRIMENET_ASSIGN_FACTOR);
	m_dblchk = !! (work_pref & PRIMENET_ASSIGN_DBLCHK);
	m_new_dates = 0;

	askYN ("Use PrimeNet to get work and report results", &m_primenet);
	if (!m_primenet) goto done;
	askYN ("Use a dial-up connection to the Internet", &m_dialup);
	askNum ("Days of work to get", &m_work, 1, 90);
	askYN ("Request whatever type of work makes the most sense",
		&m_work_dflt);
	if (!m_work_dflt) {
		askYN ("Request Mersenne numbers to run primality tests",
			&m_lucas);
		askYN ("Request Mersenne numbers to factor", &m_factor);
		askYN ("Request Mersenne numbers to double-check", &m_dblchk);
	}
	askYN ("Send new completion dates to PrimeNet server", &m_new_dates);

done:	if (askOkCancel ()) {
		if (!USE_PRIMENET && m_primenet) {
			USE_PRIMENET = 1;
			spoolMessage (PRIMENET_MAINTAIN_USER_INFO, NULL);
			spoolMessage (PRIMENET_SET_COMPUTER_INFO, NULL);
			spoolExistingResultsFile ();
		}
		USE_PRIMENET = m_primenet;
		DIAL_UP = m_dialup;
		DAYS_OF_WORK = m_work;
		if (m_work_dflt)
			WORK_PREFERENCE = 0;
		else
			WORK_PREFERENCE =
				(m_lucas ? PRIMENET_ASSIGN_TEST : 0) +
				(m_factor ? PRIMENET_ASSIGN_FACTOR : 0) +
				(m_dblchk ? PRIMENET_ASSIGN_DBLCHK : 0);
		IniWriteInt (INI_FILE, "UsePrimenet", USE_PRIMENET);
		IniWriteInt (INI_FILE, "DialUp", DIAL_UP);
		IniWriteInt (INI_FILE, "DaysOfWork", DAYS_OF_WORK);
		IniWriteInt (INI_FILE, "WorkPreference", WORK_PREFERENCE);
		if (m_new_dates) UpdateEndDates ();
		CHECK_WORK_QUEUE = 1;
		if (STARTUP_IN_PROGRESS) {
			STARTUP_IN_PROGRESS = 0;
			if (USE_PRIMENET) primeContinue ();
		}
	} else
		STARTUP_IN_PROGRESS = 0;
}

/* Test/Manual Operation dialog */

test_manual_op ()
{
	unsigned long m_start, m_end, m_work;
	short	work_preference;

	m_start = 0;
	m_end = 0;
	work_preference = default_work_type ();
	if (work_preference == PRIMENET_ASSIGN_TEST) m_work = 0;
	else m_work = 1;

	m_work++;
	askNum ("Type of Work to do, 1=Lucas-Lehmer Primality Tests\n\t2=Factoring only 3=Double-check", &m_work, 1, 3);
	m_work--;

	askNumNoDflt ("Range start", &m_start, 0, 0);
	askNumNoDflt ("Range end", &m_end, 0, 0);

	if (askOkCancel ()) {
		getWorkFromDatabase (m_start, m_end, m_work == 1, m_work == 2);
		rangeStatus ();
	}
}

/* Test/User Information dialog */

void test_user ()
{
	char	m_name[80], m_email[80], m_userid[20];
	char	m_password[20], m_compid[20];
	int	m_sendemail;

	if (IniGetInt (INI_FILE, "LockUserInfo", 0)) {
		OutputStr ("The user information cannot be changed.\n");
		askOK ();
		return;
	}   

	strcpy (m_name, USER_NAME);
	strcpy (m_email, USER_ADDR);
	strcpy (m_userid, USERID);
	strcpy (m_password, USER_PWD);
	strcpy (m_compid, COMPID);
	m_sendemail = SEND_EMAIL;

	askStr ("Your name", m_name, 76);
	askStr ("Your email address", m_email, 76);
	askYN ("Receive email from PrimeNet server if exponents are about to expire", &m_sendemail);
	outputLongLine ("You can either pick your own user ID and password or let the PrimeNet server assign one to you.  See the readme.txt file for more details.");
	askStr ("Your user ID", m_userid, 14);
	askStr ("Your password", m_password, 8);
	askStr ("Optional Computer ID", m_compid, 12);

	if (askOkCancel ()) {
		if (strcmp (COMPID, m_compid) != 0) {
			strcpy (COMPID, m_compid);
			sanitizeString (COMPID);
			IniWriteString (LOCALINI_FILE, "ComputerID", COMPID);
			spoolMessage (PRIMENET_SET_COMPUTER_INFO, NULL);
		}
		if (OLD_USERID[0] == 0 &&
		    strcmp (USERID, m_userid) != 0) {
			strcpy (OLD_USERID, USERID);
			strcpy (OLD_USER_PWD, USER_PWD);
			IniWriteString (INI_FILE, "OldUserID", OLD_USERID);
			IniWriteString (INI_FILE, "OldUserPWD", OLD_USER_PWD);
		}
		strcpy (USER_PWD, m_password);
		sanitizeString (USER_PWD);
		IniWriteString (INI_FILE, "UserPWD", USER_PWD);
		if (strcmp (USER_NAME, m_name) != 0 ||
		    strcmp (USER_ADDR, m_email) != 0 ||
		    SEND_EMAIL != m_sendemail ||
		    strcmp (USERID, m_userid) != 0) {
			strcpy (USER_NAME, m_name);
			strcpy (USER_ADDR, m_email);
			strcpy (USERID, m_userid);
			sanitizeString (USERID);
			SEND_EMAIL = m_sendemail;
			IniWriteString (INI_FILE, "UserName", USER_NAME);
			IniWriteString (INI_FILE, "UserEmailAddr", USER_ADDR);
			IniWriteInt (INI_FILE, "SendEmail", SEND_EMAIL);
			IniWriteString (INI_FILE, "UserID", USERID);
			spoolMessage (PRIMENET_MAINTAIN_USER_INFO, NULL);
		}
		if (STARTUP_IN_PROGRESS) options_cpu ();
	} else
		STARTUP_IN_PROGRESS = 0;
}

/* Test/Vacation or Holiday dialog */

void test_vacation ()
{
	unsigned long m_vacation_days;
	int	m_computer_on;

	m_vacation_days = (secondsUntilVacationEnds () + 43200) / 86400;
	m_computer_on = ON_DURING_VACATION;

	outputLongLine ("If you are going on vacation and your computer will not be running or your computer will not be connected to the Internet, then the PrimeNet server must be informed of new expected completion dates.");
	outputLongLine ("\nPlease make sure you are connected to the Internet long enough to send the new completion dates.\n");
	askNum ("Days of vacation", &m_vacation_days, 0, 120);
	askYN ("Computer will be on during your absence", &m_computer_on);

	if (askOkCancel ()) {
		if (m_vacation_days) {
			time (&VACATION_END);
			VACATION_END += m_vacation_days * 86400;
		} else
			VACATION_END = 0;
		ON_DURING_VACATION = m_computer_on;
                IniWriteInt (LOCALINI_FILE, "VacationEnd", VACATION_END);
                IniWriteInt (LOCALINI_FILE, "VacationOn", ON_DURING_VACATION);
		if (VACATION_END && !ON_DURING_VACATION)
			IniWriteInt (LOCALINI_FILE, "RollingStartTime", 0);
		UpdateEndDates ();
		MANUAL_COMM |= 0x2;
		CHECK_WORK_QUEUE = 1;
		communicateWithServer ();
	}
}

/* Output a status report for the range */

void rangeStatus ()
{
	char	buf[2000];

	rangeStatusMessage (buf);
	outputLongLine (buf);
	askOK ();
}

/* Advanced/Test dialog */

void advanced_test ()
{
	unsigned long m_p;
#define NOTPRIMEERR "This number is not prime, there is no need to test it.\n"

loop:	m_p = 0;
	askNumNoDflt ("Exponent to test", &m_p, MIN_PRIME, MAX_PRIME);

	if (askOkCancel ()) {
		if (! isPrime (m_p)) {
			printf (NOTPRIMEERR);
			goto loop;
		}
		IniFileOpen (WORKTODO_FILE, 1);
		IniInsertLineAsInt (WORKTODO_FILE, 1, "AdvancedTest", m_p);
		primeContinue ();
		askOK ();
	}
}

/* Advanced/Time dialog */

void advanced_time ()
{
	unsigned long m_p, m_iter;

	m_p = 900000;
	m_iter = 10;

	askNum ("Exponent to time", &m_p, MIN_PRIME, MAX_PRIME);
	askNum ("Number of Iterations", &m_iter, 1, 1000);
	if (askOkCancel ()) {
		primeTime (m_p, m_iter);
		askOK ();
	}
}

/* Advanced/ECM dialog */

void advanced_ecm ()
{
	unsigned long m_p, m_bound1, m_bound2, m_num_curves;
	int	m_plus1;
	double	m_curve;

	m_p = 0;
	m_bound1 = 1000000;
	m_bound2 = 0;
	m_curve = 0.0;
	m_num_curves = 100;
	m_plus1 = 0;

	askNumNoDflt ("Exponent", &m_p, 100, 22831);
	askNum ("Bound #1", &m_bound1, 100, 1000000000);
	askNum ("Bound #2", &m_bound2, 0, 4000000000);
	askNum ("Curves to test", &m_num_curves, 1, 100000);
	askYN ("Test 2^N+1", &m_plus1);

	if (askOkCancel ()) {
		struct work_unit w;
		w.work_type = WORK_ECM; 
		w.p = m_p;
		w.B1 = m_bound1;
		w.B2 = m_bound2;
		w.curves_to_do = m_num_curves;
		w.curves_completed = 0;
		w.curve = m_curve;
		w.plus1 = m_plus1;
		addWorkToDoLine (&w);
		primeContinue ();
		askOK ();
	}
}

/* Advanced/Priority dialog */

void advanced_priority ()
{
	unsigned long m_priority;

	m_priority = PRIORITY;

	outputLongLine ("Pick a priority between 1 and 10 where 1 is the lowest priority and 10 is the highest.");
	outputLongLine ("It is strongly recommended that you use the default priority of 1.  Your throughput will probably not improve by using a higher priority.");
	outputLongLine ("The only time you should raise the priority is when another process, such as a screen saver, is stealing CPU cycles from this program.");
	askNum ("Priority", &m_priority, 1, 10);

	if (askOkCancel ()) {
		PRIORITY = m_priority;
		IniWriteInt (INI_FILE, "Priority", PRIORITY);
	}
}

/* Advanced/Manual Communication dialog */

void advanced_manualcomm ()
{
	int	m_manual_comm, m_comm_now;

	m_manual_comm = MANUAL_COMM & 0x1;
	m_comm_now = 1;

	m_manual_comm = !m_manual_comm;
	askYN ("Contact PrimeNet server automatically", &m_manual_comm);
	m_manual_comm = !m_manual_comm;
	askYN ("Contact PrimeNet server now", &m_comm_now);

	if (askOkCancel ()) {
		MANUAL_COMM = m_manual_comm;
		IniWriteInt (INI_FILE, "ManualComm", MANUAL_COMM);
		if (m_comm_now) {
			MANUAL_COMM |= 0x2;
			CHECK_WORK_QUEUE = 1;
			communicateWithServer ();
		}
	}
}

/* Advanced/Quit Gimps dialog */

void advanced_quit ()
{

	if (!USE_PRIMENET) {
		outputLongLine (MANUAL_QUIT);
		if (askOkCancel ()) {
			writeResults ("Quitting GIMPS.\n");
			IniDeleteAllLines (WORKTODO_FILE);
		}
	} else {
		outputLongLine (PRIMENET_QUIT);
		if (askOkCancel ()) {
			writeResults ("Quitting GIMPS.\n");
			spoolMessage (999, NULL);
			MANUAL_COMM |= 0x2;
			CHECK_WORK_QUEUE = 1;
			communicateWithServer ();
			askOK ();
		}
	}
}

/* Options/CPU dialog */

void options_cpu ()
{
	unsigned long m_cpu_type, m_speed, m_hours;
	int	limit;

	m_cpu_type =
		(CPU_TYPE == 3) ? 0 : (CPU_TYPE == 4) ? 1 :
		(CPU_TYPE == 5) ? 3 : (CPU_TYPE == 6) ? 4 : 2;
	m_speed = CPU_SPEED;
	m_hours = CPU_HOURS;

	m_cpu_type++;
	if (isPentiumPro ()) limit = 5;
	else if (isPentium ()) limit = 4;
	else limit = 2;
	askNum ("CPU Type, 1=Cyrix 2=486 3=AMD K6 4=Pentium 5=Pentium Pro or Pentium II",
		&m_cpu_type, 1, limit);
	m_cpu_type--;
	askNum ("CPU speed in MHz", &m_speed, 25, 1000);
	askNum ("Hours per day this program will run", &m_hours, 1, 24);

	if (askOkCancel ()) {
		unsigned int new_cpu_type;

		new_cpu_type = (m_cpu_type == 0) ? 3 :
			       (m_cpu_type == 1) ? 4 :
			       (m_cpu_type == 3) ? 5 :
			       (m_cpu_type == 4) ? 6 : 7;
		if (CPU_SPEED != m_speed ||
		    CPU_TYPE != new_cpu_type ||
		    CPU_HOURS != m_hours) {
			ROLLING_AVERAGE = 1000;
			IniWriteInt (LOCALINI_FILE, "RollingAverage", 1000);
			IniWriteInt (LOCALINI_FILE, "RollingStartTime", 0);
			spoolMessage (PRIMENET_SET_COMPUTER_INFO, NULL);
			UpdateEndDates ();
		}
		CPU_SPEED = m_speed;
		CPU_TYPE = new_cpu_type;
		CPU_HOURS = m_hours;
		IniWriteInt (LOCALINI_FILE, "CPUType", CPU_TYPE);
		IniWriteInt (LOCALINI_FILE, "CPUSpeed", CPU_SPEED);
		IniWriteInt (LOCALINI_FILE, "CPUHours", CPU_HOURS);
		if (STARTUP_IN_PROGRESS) test_primenet ();
	} else
		STARTUP_IN_PROGRESS = 0;
}

/* Options/Preferences dialog */

void options_preferences ()
{
	unsigned long m_iter, m_r_iter, m_disk_write_time;
	unsigned long m_modem, m_retry, m_end_dates;
	int	m_backup, m_noise;

	m_iter = ITER_OUTPUT;
	m_r_iter = ITER_OUTPUT_RES;
	m_disk_write_time = DISK_WRITE_TIME;
	m_modem = MODEM_RETRY_TIME;
	m_retry = NETWORK_RETRY_TIME;
	m_end_dates = DAYS_BETWEEN_CHECKINS;
	m_backup = TWO_BACKUP_FILES;
	m_noise = !SILENT_VICTORY;

	askNum ("Iterations between screen outputs", &m_iter, 1, 999999);
	askNum ("Iterations between results file outputs",
		&m_r_iter, 100000, 99999999);
	askNum ("Minutes between disk writes", &m_disk_write_time, 10, 999999);
	if (PRIMENET)
		askNum ("Minutes between modem retries", &m_modem, 1, 300);
	if (PRIMENET)
		askNum ("Minutes between network retries", &m_retry, 1, 300);
	if (PRIMENET)
		askNum ("Days between sending end dates", &m_end_dates, 1, 60);
	askYN ("Create Two Backup Files", &m_backup);
	askYN ("Make noise if new Mersenne prime is found", &m_noise);

	if (askOkCancel ()) {
		ITER_OUTPUT = m_iter;
		ITER_OUTPUT_RES = m_r_iter;
		DISK_WRITE_TIME = m_disk_write_time;
		NETWORK_RETRY_TIME = m_retry;
		DAYS_BETWEEN_CHECKINS = m_end_dates;
		TWO_BACKUP_FILES = m_backup;
		SILENT_VICTORY = !m_noise;
		IniWriteInt (INI_FILE, "OutputIterations", ITER_OUTPUT);
		IniWriteInt (INI_FILE, "ResultsFileIterations", ITER_OUTPUT_RES);
		IniWriteInt (INI_FILE, "DiskWriteTime", DISK_WRITE_TIME);
		IniWriteInt (INI_FILE, "ModemRetryTime", MODEM_RETRY_TIME);
		IniWriteInt (INI_FILE, "NetworkRetryTime", NETWORK_RETRY_TIME);
		IniWriteInt (INI_FILE, "DaysBetweenCheckins", DAYS_BETWEEN_CHECKINS);
		IniWriteInt (INI_FILE, "TwoBackupFiles", TWO_BACKUP_FILES);
		IniWriteInt (INI_FILE, "SilentVictory", SILENT_VICTORY);
	}
}

/* Help/About */

void help_about ()
{
	printf ("Mersenne Prime Finder - Version %s.2\n", VERSION);
	printf ("Copyright 1996-1998 Just For Fun Software, Inc.\n");
	printf ("Author: George Woltman\n");
	printf ("Email:  woltman@magicnet.net\n");
	askOK ();
}

/* Help/About PrimeNet Server */

void help_about_server ()
{
	struct primenetPingServerInfo pkt;

	if (sendMessage (PRIMENET_PING_SERVER_INFO, &pkt)) {
		outputLongLine (PING_ERROR);
	} else {
		printf ("Version: %s\n", pkt.buildID);
		printf ("Server Name: %s\n", pkt.primenetServerName);
		printf ("Email Addr: %s\n", pkt.adminEmailAddr);
	}
	askOK ();
}

/* Display the main menu */

void main_menu ()
{
	unsigned long choice;

mloop:	if (THREAD_KILL) return;
	printf ("\t     Main Menu\n");
loop:	printf ("\n");
	printf ("\t 1.  Test/Primenet\n");
	printf ("\t 2.  Test/Manual Operation\n");
	printf ("\t 3.  Test/User Information\n");
	printf ("\t 4.  Test/Vacation or Holiday\n");
	printf ("\t 5.  Test/Status\n");
	printf ("\t 6.  Test/Continue\n");
	printf ("\t 7.  Test/Exit\n");
	printf ("\n");
	printf ("\t 8.  Advanced/Test\n");
	printf ("\t 9.  Advanced/Time\n");
	printf ("\t10.  Advanced/ECM\n");
	printf ("\t11.  Advanced/Priority\n");
	printf ("\t12.  Advanced/Manual Communication\n");
	printf ("\t13.  Advanced/Quit Gimps\n");
	printf ("\n");
	printf ("\t14.  Options/CPU\n");
	printf ("\t15.  Options/Preferences\n");
	printf ("\t16.  Options/Self Test\n");
	printf ("\t17.  Options/Torture Test\n");
	printf ("\n");
	printf ("\t18.  Help/About\n");
	printf ("\t19.  Help/About PrimeNet Server\n");
	printf ("\n");
	printf ("Your choice: ");
	choice = get_number (0);
	if (choice <= 0 || choice >= 20) {
		printf ("\t     Invalid choice\n");
		goto loop;
	}

/* Display the main menu and switch off the users choice */

	printf ("\n");
	switch (choice) {

/* Test/Primenet dialog */

	case 1:
		test_primenet ();
		break;

/* Test/Manual Operation dialog */

	case 2:
		test_manual_op ();
		break;

/* Test/User Information dialog */

	case 3:
		test_user ();
		break;

/* Test/Vacation or Holiday dialog */

	case 4:
		test_vacation ();
		break;

/* Test/Status message */

	case 5:
		rangeStatus ();
		break;

/* Test/Continue */

	case 6:
		primeContinue ();
		askOK ();
		break;

/* Test/Exit */

	case 7:
		return;

/* Advanced/Test dialog */

	case 8:
		advanced_test ();
		break;

/* Advanced/Time dialog */

	case 9:
		advanced_time ();
		break;

/* Advanced/ECM dialog */

	case 10:
		advanced_ecm ();
		break;

/* Advanced/Priority dialog */

	case 11:
		advanced_priority ();
		break;

/* Advanced/Manual Communication dialog */

	case 12:
		advanced_manualcomm ();
		break;

/* Advanced/Quit Gimps dialog */

	case 13:
		advanced_quit ();
		break;

/* Options/CPU dialog */

	case 14:
		options_cpu ();
		break;

/* Options/Preferences dialog */

	case 15:
		options_preferences ();
		break;

/* Options/Self Test */

	case 16:
		selfTest (0);
		askOK ();
		break;

/* Options/Torture Test */

	case 17:
		selfTest (1);
		askOK ();
		break;

/* Help/About */

	case 18:
		help_about ();
		break;

/* Help/About PrimeNet Server */

	case 19:
		help_about_server ();
		break;
	}
	goto mloop;
}
