/* Copyright 1995-1998 Just For Fun Software, Inc. */
/* Author:  George Woltman */
/* Email: woltman@magicnet.net */

/* Reserve lots of memory, say 16MB */

double mem1[131072];
double mem2[131072];
double mem3[131072];
double mem4[131072];
double mem5[131072];
double mem6[131072];
double mem7[131072];
double mem8[131072];
double mem1a[131072];
double mem2a[131072];
double mem3a[131072];
double mem4a[131072];
double mem5a[131072];
double mem6a[131072];
double mem7a[131072];
double mem8a[131072];

/* Include files */

# include <ctype.h>

#ifdef __FreeBSD__
/* FreeBSD needs to process sys/types.h before it can understand either
/* sys/time.h or sys/resource.h */
# include <sys/types.h>
/* This is an ugly hack -- "_U" is defined in both ctype.h and */
/* prime95/ecmhelp.asm, so it's necessary to undefine it here. */
/* This will be fixed in a later release */
# undef _U
#endif

#if defined (__linux__) || defined (__FreeBSD__)
# include <sys/time.h>
#endif

#include <sys/resource.h>

#include "prime.h"

#include <dirent.h>
#include <fcntl.h>
#include <math.h>
#include <memory.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/* FreeBSD also cannot be built on systems pre v3.0 due to elf */
/* compatibility issues. */
#ifdef __FreeBSD__
# include <osreldate.h>
# if __FreeBSD_version < 300000
#  error "Cannot build mprime on FreeBSD < 3.0"
# endif
#endif

/* Globals */

#define LINUX_LDAV_FILE "/proc/loadavg"
int volatile SLEEP_STOP = 0;
long LOAD_CHECK_TIME = 0;
double HI_LOAD = 0.0;
double LO_LOAD = 0.0;

int volatile THREAD_STOP = 0;
int volatile THREAD_KILL = 0;
int NO_GUI = 1;
int VERBOSE = 0;
int MENUING = 0;

/* Common code */

#define PORT	2
#include "commona.c"
#include "commonb.c"
#include "commonc.c"
#include "ecm.c"

/* Signal handlers */

void sigterm_handler(int signo)
{
	THREAD_STOP = TRUE;
	if (signo != SIGINT) THREAD_KILL = TRUE;
	(void)signal(signo, sigterm_handler);
}

#ifdef MPRIME_LOADAVG

double
get_load_average()
{
#ifdef __linux__
	char	ldavgbuf[40];
	double	load_avg;
	int	fd, count;

	fd = open (LINUX_LDAV_FILE, O_RDONLY);
	if (fd == -1)
		return;
	count = read (fd, ldavgbuf, 40);
	(void) close (fd);
	if (count <= 0)
		return;

	count = sscanf (ldavgbuf, "%lf", &load_avg);
	if (count < 1)
		return;

#else
# ifdef __FreeBSD__
	double load[3];

	if (getloadavg (load, sizeof(load)/sizeof(load[0])) < 0)
		return -1;
	else
		return(load[0]);
# else
#  error "mprime is configured to compute loadavg, but no code exists to do it"
# endif	/* __FreeBSD__ */
#endif	/* __linux__ */
}

/* load_handler: call by signal routine,
   sets SLEEP_STOP to TRUE if load is too high */
void load_handler (
	int	sig)
{
	double  load_avg;
	if ((load_avg = get_load_average()) < 0)
		return;

	if (SLEEP_STOP) {
		if (load_avg < LO_LOAD)
			SLEEP_STOP = FALSE;
	} else {
		if (load_avg > HI_LOAD)
			SLEEP_STOP = TRUE;
	}
}

/* init_load_check: initialises timer that calls load_handler
   every LOAD_CHECK_TIME seconds */
void init_load_check ()
{
	struct itimerval timer, otimer;
	struct sigaction sigact;
	int	ret;

	timer.it_interval.tv_sec  =  LOAD_CHECK_TIME;
	timer.it_interval.tv_usec =  0;
	timer.it_value.tv_sec     =  LOAD_CHECK_TIME;
	timer.it_value.tv_usec    =  0;

	ret = setitimer (ITIMER_REAL, &timer, &otimer);
	if (ret < 0) return;
  
	sigact.sa_handler = &load_handler;
	sigemptyset(&sigact.sa_mask);
	sigact.sa_flags =  SA_RESTART;
	ret = sigaction(SIGALRM, &sigact, NULL);
	if (ret < 0) { /* clean up after ourselves */
		setitimer (ITIMER_REAL, &otimer, NULL);
	}
}

/* test_sleep: tests if SLEEP_STOP is set and sleeps until load is normal
   again or THREAD_STOP is set
*/
void test_sleep (void) 
{
	sigset_t newmask;

	while (SLEEP_STOP && !THREAD_STOP) {
		sigemptyset (&newmask);
		sigsuspend (&newmask);
	}
}

#endif /* MPRIME_LOADAVG */


/* Main entry point! */

int main (
	int	argc,
	char	*argv[])
{
	char	buf[256];
	int	named_ini_files = -1;
	int	i, self_test, range_status;
	char	*p;

/* catch termination signals */

	(void)signal(SIGTERM, sigterm_handler);
	(void)signal(SIGINT, sigterm_handler);

/* No buffering of output */

	setvbuf (stdout, NULL, _IONBF, 0);

/* Change to the executable's directory*/

	strcpy (buf, argv[0]);
	p = strrchr (buf, '/');
	if (p != NULL) {
		*p = 0;
		_chdir (buf);
	}

/* Process command line switches */

	for (i = 1; i < argc; i++) {
		p = argv[i];

		if (*p++ != '-') break;
		switch (*p++) {

/* Accept a -A switch indicating an alternate set of INI files */
/* are to be used. */

		case 'A':
		case 'a':
			named_ini_files = 0;
			while (isspace (*p)) p++;
			while (isdigit (*p)) {
				named_ini_files = named_ini_files * 10 + (*p - '0');
				p++;
			}
			break;

/* -D - debug */

		case 'D':
		case 'd':
			VERBOSE = TRUE;
			NO_GUI = FALSE;
			break;

/* -H - help */

		case 'H':
		case 'h':
		case '?':
			goto usage;

/* -M - Menu */

		case 'M':
		case 'm':
			MENUING = TRUE;
			NO_GUI = FALSE;
			break;

/* -V - version number */

		case 'V':
		case 'v':
			printf ("Mersenne Prime Test Program, Version %s.2\n", VERSION);
			return (0); 

/* -W - use a different working directory */

		case 'W':
		case 'w':
			_chdir (p);
			break; 

/* Otherwise unknown switch */

		default:
			printf ("Invalid switch\n");
			goto usage;
		}
	}

/* Determine the names of the INI files */
/* Read the INI files */

	nameIniFiles (named_ini_files);
	readIniFiles ();

/* Read Unix specific settings from INI files */

	IniGetString (INI_FILE, "MaxLoad", buf, sizeof (buf), "0");
	HI_LOAD = atof (buf);
	IniGetString (INI_FILE, "MinLoad", buf, sizeof (buf), "0");
	LO_LOAD = atof (buf);
	IniGetString (INI_FILE, "PauseTime", buf, sizeof (buf), "0");
	LOAD_CHECK_TIME = atol (buf);

/* Initialise load checking */

#ifdef MPRIME_LOADAVG
	if (HI_LOAD > 0.0 && LOAD_CHECK_TIME > 0)
		init_load_check ();
#endif

/* Bring up the main menu */

	if (MENUING)
		main_menu ();

/* On first run, get user name and email address */
/* before contacting server for a work assignment */

	else if (USE_PRIMENET && USERID[0] == 0) {
		STARTUP_IN_PROGRESS = 1;
		test_user ();
	}

/* Continue testing the range */

	else
		primeContinue ();

/* All done */

	return (0);

/* Invalid args message */

usage:	printf ("Usage: mprime [-aN] [-dhmv] [-wDIR]\n");
	printf ("-aN\tUse an alternate set of INI and output files.\n");
	printf ("-d\tPrint detailed information to stdout.\n");
	printf ("-h\tPrint this.\n");
	printf ("-m\tMenu to configure mprime.\n");
	printf ("-v\tPrint the version number.\n");
	printf ("-wDIR\tRun from a different working directory.\n");
	printf ("\n");
	return (1);
}


/* Reserve memory for the factoring code */

int loadfac ()
{
	THE_END = DATA_ADDR + 0x00100000;	/* 1MB should be plenty */
	return (TRUE);
}

/* Choose the correct LL routine */

int loadRightExe (
	unsigned long p,
	unsigned long *fftlen)
{

/* Find the best fft length */

	if (*fftlen == 0) *fftlen = pick_fft_length (p);

/* The fft length determines the addresses. */

	if (*fftlen == 1048576) THE_END = THE_END_1024, lucas = lucas1024;
	if (*fftlen == 917504) THE_END = THE_END_896, lucas = lucas896;
	if (*fftlen == 786432) THE_END = THE_END_768, lucas = lucas768;
	if (*fftlen == 655360) THE_END = THE_END_640, lucas = lucas640;
	if (*fftlen == 524288) THE_END = THE_END_512, lucas = lucas512;
	if (*fftlen == 458752) THE_END = THE_END_448, lucas = lucas448;
	if (*fftlen == 393216) THE_END = THE_END_384, lucas = lucas384;
	if (*fftlen == 327680) THE_END = THE_END_320, lucas = lucas320;
	if (*fftlen == 262144) THE_END = THE_END_256, lucas = lucas256;
	if (*fftlen == 229376) THE_END = THE_END_224, lucas = lucas224;
	if (*fftlen == 196608) THE_END = THE_END_192, lucas = lucas192;
	if (*fftlen == 163840) THE_END = THE_END_160, lucas = lucas160;
	if (*fftlen == 131072) THE_END = THE_END_128, lucas = lucas128;
	if (*fftlen == 114688) THE_END = THE_END_112, lucas = lucas112;
	if (*fftlen == 98304) THE_END = THE_END_96, lucas = lucas96;
	if (*fftlen == 81920) THE_END = THE_END_80, lucas = lucas80;

/* Done */

	return (TRUE);
}

/* Load the FFT data and call setup */

void lucasSetup (
	unsigned long p,
	unsigned long fftlen)
{
	PARG = p;
	if (fftlen == 1048576) setup1024 ();
	if (fftlen == 917504) setup896 ();
	if (fftlen == 786432) setup768 ();
	if (fftlen == 655360) setup640 ();
	if (fftlen == 524288) setup512 ();
	if (fftlen == 458752) setup448 ();
	if (fftlen == 393216) setup384 ();
	if (fftlen == 327680) setup320 ();
	if (fftlen == 262144) setup256 ();
	if (fftlen == 229376) setup224 ();
	if (fftlen == 196608) setup192 ();
	if (fftlen == 163840) setup160 ();
	if (fftlen == 131072) setup128 ();
	if (fftlen == 114688) setup112 ();
	if (fftlen == 98304) setup96 ();
	if (fftlen == 81920) setup80 ();
}

void title (char *msg)
{
}

void flashWindowAndBeep ()
{
	printf ("\007");
}

/* Return TRUE if we should stop calculating */

int escapeCheck ()
{
	if (THREAD_STOP) {
		THREAD_STOP = 0;
		return (TRUE);
	}
	return (FALSE);
}

void doMiscTasks ()
{
#ifdef MPRIME_LOADAVG
	test_sleep ();
#endif
}

void OutputStr (char *buf)
{
	if (VERBOSE || MENUING) fputs (buf, stdout);
}

void guessCpuType ()
{
	CPU_TYPE = 5;
	CPU_SPEED = 100;
}

int Sleep (
	long	ms) 
{
	sleep (ms/1000);
}

/* Set priority.  Map one (prime95's lowest priority) to 20 */
/* (unix's lowest priority).  Map eight (prime95's normal priority) to */
/* 0 (unix's normal priority). */

void SetPriority ()
{
	int	p;
	p = (8 - (int) PRIORITY) * 20 / 7;
	setpriority (PRIO_PROCESS, getpid (), p);
}

