//
// Copyright 1995-1997 - Just For Fun Software, Inc., all rights reserved
// Author:  George Woltman
// Email: woltman@magicnet.net
//


/****************************************************************************

    PROGRAM: main.c

    PURPOSE: prime template for Windows applications

    FUNCTIONS:

	WinMain() - calls initialization function, processes message loop
	InitApplication() - initializes window data and registers window
	InitInstance() - saves instance handle and creates main window
	MainWndProc() - processes messages
	About() - processes messages for "About" dialog box

    COMMENTS:

        Windows can have several copies of your application running at the
        same time.  The variable hInst keeps track of which instance this
        application is so that processing will be to the correct window.

****************************************************************************/

#include "prime.h"	    /* specific to this program		     */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

HANDLE	NEAR hInst = 0;	    /* current instance			     */
HWND	NEAR hMainWnd = 0;
WORD	NEAR DataSel = 0;
WORD	NEAR CodeSel = 0;

HMODULE HLIB = 0;
int (CALLBACK *PRIMENET)(short, void *) = 0;

// Abort support.
BOOL	NEAR bAbort;
HWND	NEAR hDlgAbort;
FARPROC lpfnAbortDlgProc;

// Window text
#define NumLines	32
#define LineSize	128
static  char *	NEAR lines[NumLines] = {0};
static  char	NEAR linebuf[NumLines*LineSize];
static	int	NEAR charHeight = 0;

// Routine to enabled the Advanced Menu

void enable_advanced (HWND hWnd)
{
	EnableMenuItem (GetSubMenu (GetMenu (hWnd), 1),
		        IDM_PASSWORD,
			ADVANCED_ENABLED ? MF_GRAYED : MF_ENABLED);
	EnableMenuItem (GetSubMenu (GetMenu (hWnd), 1),
		        IDM_START,
			ADVANCED_ENABLED ? MF_ENABLED : MF_GRAYED);
	EnableMenuItem (GetSubMenu (GetMenu (hWnd), 1),
		        IDM_TIME,
			ADVANCED_ENABLED ? MF_ENABLED : MF_GRAYED);
	EnableMenuItem (GetSubMenu (GetMenu (hWnd), 1),
		        IDM_FACTOR,
			ADVANCED_ENABLED ? MF_ENABLED : MF_GRAYED);
	EnableMenuItem (GetSubMenu (GetMenu (hWnd), 1),
		        IDM_ERRCHK,
			ADVANCED_ENABLED ? MF_ENABLED : MF_GRAYED);
}

char NEAR CAPTION[] = "Error";
char NEAR RANGEERR1[] = "The range start must be less than the range end.";
char NEAR RANGEERR2[] = "The range end must be less than %ld.";

int checkRange (
	HWND	hDlg,
	unsigned long start,
	unsigned long end,
	unsigned long fudge)
{
	if (start > end) {
		MessageBox (hDlg, RANGEERR1, CAPTION, MB_ICONHAND | MB_OK);
		return (TRUE);
	}
	if (end > MAX_PRIME + fudge) {
		char	buf[80];
		sprintf (buf, RANGEERR2, MAX_PRIME + fudge);
		MessageBox (hDlg, buf, CAPTION, MB_ICONHAND | MB_OK);
		return (TRUE);
	}
	return (FALSE);
}

/****************************************************************************

    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop

    COMMENTS:

        Windows recognizes this function by name as the initial entry point 
        for the program.  This function calls the application initialization 
        routine, if no other instance of the program is running, and always 
        calls the instance initialization routine.  It then executes a message
        retrieval and dispatch loop that is the top-level control structure 
        for the remainder of execution.  The loop is terminated when a WM_QUIT
        message is received, at which time this function exits the application
        instance by returning the value passed by PostQuitMessage(). 

        If this function must abort before entering the message loop, it 
        returns the conventional value NULL.  

****************************************************************************/

int PASCAL WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;		     /* current instance	     */
HANDLE hPrevInstance;		     /* previous instance	     */
LPSTR lpCmdLine;		     /* command line		     */
int nCmdShow;			     /* show-window type (open/icon) */
{
    MSG msg;			     /* message			     */
    HWND hWnd;

    if (!hPrevInstance)			 /* Other instances of app running? */
	if (!InitApplication(hInstance)) /* Initialize shared things */
	    return (FALSE);		 /* Exits if unable to initialize */

    /* Perform initializations that apply to a specific instance */

    if (!InitInstance(hInstance, nCmdShow, &hWnd))
        return (FALSE);

    /* Delete old-style auxillary files */

    _unlink ("lucas.exe");
    _unlink ("lucas14.exe");
    _unlink ("lucas15.exe");
    _unlink ("lucas16.exe");
    _unlink ("factor64.exe");
    _unlink ("lucas.dll");
    _unlink ("lucas14.dll");
    _unlink ("lucas15.dll");
    _unlink ("lucas16.dll");
    _unlink ("lucas17.dll");

/* Read the INI files */

	nameIniFiles (-1);
	readIniFiles ();
	USE_PRIMENET = 0;

/* Apply INI file */

    enable_advanced (hWnd);
    CheckMenuItem (GetSubMenu (GetMenu (hWnd), 1),
		    IDM_ERRCHK,
		    ERRCHK ? MF_CHECKED : MF_UNCHECKED);

    /* Unless the first argument is D (for debug), continue any previously */
    /* started run.  The user can always stop this default action by */
    /* hitting the ESCape key. */
    /* Unfortunately, escapeCheck does not work when started this way!??? */
    /* so instead we'll force the user to select Continue from the File menu */

    if (lpCmdLine[strlen(lpCmdLine)-1] == 'C')
	PostMessage (hWnd, WM_COMMAND, IDM_CONTINUE, 0L);

    /* Acquire and dispatch messages until a WM_QUIT message is received. */

    while (GetMessage(&msg,	   /* message structure */
	   NULL,		   /* handle of window receiving the msg */
	    NULL,		   /* lowest message to examine */
	    NULL))		   /* highest message to examine */
	{
	TranslateMessage(&msg);	   /* Translates virtual key codes */
	DispatchMessage(&msg);	   /* Dispatches message to window */
    }
    return (msg.wParam);	   /* Returns the value from PostQuitMessage*/
}


/****************************************************************************

    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class

    COMMENTS:

        This function is called at initialization time only if no other 
        instances of the application are running.  This function performs 
        initialization tasks that can be done once for any number of running 
        instances.  

        In this case, we initialize a window class by filling out a data 
        structure of type WNDCLASS and calling the Windows RegisterClass() 
        function.  Since all instances of this application use the same window
        class, we only need to do this when the first instance is initialized.

****************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;		       /* current instance	     */
{
    WNDCLASS  wc;

    /* Fill in window class structure with parameters that describe the */
    /* main window.                                                     */

    wc.style = NULL;                    /* Class style(s). */
    wc.lpfnWndProc = MainWndProc;       /* Function to retrieve messages for*/
                                        /* windows of this class. */
    wc.cbClsExtra = 0;                  /* No per-class extra data. */
    wc.cbWndExtra = 0;                  /* No per-window extra data. */
    wc.hInstance = hInstance;           /* Application that owns the class. */
    wc.hIcon = LoadIcon(NULL, (LPCSTR) "PRIME");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH); 
    wc.lpszMenuName =  "PrimeMenu";   /* Name of menu resource in .RC file. */
    wc.lpszClassName = "primeWClass"; /* Name used in call to CreateWindow. */

    /* Register the window class and return success/failure code. */

    return (RegisterClass(&wc));

}


/****************************************************************************

    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window

    COMMENTS:

        This function is called at initialization time for every instance of 
        this application.  This function performs initialization tasks that 
        cannot be shared by multiple instances.  

        In this case, we save the instance handle in a static variable and 
        create and display the main program window.  
        
****************************************************************************/

BOOL InitInstance(hInstance, nCmdShow, rethwnd)
	HANDLE	hInstance;          /* Current instance identifier. */
	int	nCmdShow;           /* Param for first ShowWindow() call. */
	HWND	FAR *rethwnd;
{
	HWND	hWnd;               /* Main window handle */
    
/* Save the instance handle in static variable, which will be used in  */
/* many subsequence calls from this application to Windows. */

	hInst = hInstance;

/* Create a main window for this application instance. */

	hWnd = CreateWindow(
		"primeWClass",		/* See RegisterClass() call. */
		"Lucas-Lehmer Test",	/* Text for window title bar. */
		WS_OVERLAPPEDWINDOW,	/* Window style. */
		CW_USEDEFAULT,		/* Default horizontal position. */
		CW_USEDEFAULT,		/* Default vertical position. */
		CW_USEDEFAULT,		/* Default width. */
		CW_USEDEFAULT,		/* Default height. */
		NULL,			/* Overlapped windows have no parent*/
		NULL,			/* Use the window class menu */
		hInstance,		/* This instance owns this window */
		NULL);			/* Pointer not needed */
	*rethwnd = hWnd;

/* If window could not be created, return "failure" */

	if (!hWnd) return (FALSE);
	hMainWnd = hWnd;
	OutputStr ("\n");

/* Make the window visible; update its client area; and return "success" */

	ShowWindow (hWnd, nCmdShow);	// Show the window
	UpdateWindow (hWnd);		// Sends WM_PAINT message
	return (TRUE);			// Returns value from PostQuitMessage
}

int FAR PASCAL reserveMem ()
{
	freeMem ();
	GlobalCompact ((DWORD) -1L);
	if (Global32Alloc (THE_END, &DataSel, THE_END, 0)) {
		OutputStr ("Out of Memory\n");
		return (FALSE);
	}

	if (Global32CodeAlias(DataSel, &CodeSel, 0)) {
		Global32Free(DataSel, 0);
		OutputStr("Out of Memory\n");
		return FALSE;
	}

	return TRUE;
}

void FAR PASCAL freeMem ()
{
	if (DataSel) {
		Global32CodeAliasFree (DataSel, CodeSel, 0);
		Global32Free (DataSel, 0);
		GlobalCompact ((DWORD) -1L);
		DataSel = 0;
		CodeSel = 0;
	}
}

int FAR PASCAL init32 ()
{
	// Freeze main window, display abort box
	EnableWindow (hMainWnd, FALSE);
	bAbort = FALSE;
	lpfnAbortDlgProc = MakeProcInstance((FARPROC)AbortDlgProc, hInst);
	hDlgAbort = CreateDialog(hInst, "AbortBox", hMainWnd, lpfnAbortDlgProc);
	if (!hDlgAbort) {
		OutputStr("Dialog error\n");
		return FALSE;
	}
	return TRUE;
}

void FAR PASCAL term32 ()
{
	// Freemem just in case an error left it allocated
	freeMem ();
	// Reactivate main window
	EnableWindow(hMainWnd, TRUE);
	DestroyWindow(hDlgAbort);

	FreeProcInstance(lpfnAbortDlgProc);
	lpfnAbortDlgProc = NULL;
	hDlgAbort = 0;
}

// Output a status report

void rangeStatus ()
{
	char	buf[2000];

	rangeStatusMessage (buf);
	MessageBox (hMainWnd, buf, "Status", MB_ICONINFORMATION | MB_OK);
}

void startRange ()
{
	rangeStatus ();
	if (init32 ()) {
		primeContinue ();
		term32 ();
	}
}

/****************************************************************************/
/*			Command Packet Definitions			    */
/****************************************************************************/

typedef struct sp {
	int	cancelled;		/* TRUE if dialog was cancelled */
	long	p;			/* Prime to test */
} start_packet;

typedef struct tp {
	int	cancelled;		/* TRUE if dialog was cancelled */
	long	p;			/* Prime to test */
	long	iterations;		/* Iterations to time */
} time_packet;

typedef struct fp {
	int	cancelled;		/* TRUE if dialog was cancelled */
	long	p1;			/* First Prime to test */
	long	p2;			/* Last Prime to test */
	int	b1;			/* Num bits in first factor */
	int	b2;			/* Num bits in last factor */
	int	ff;			/* From File - only test from primes */
} factor_packet;

typedef struct cpup {
	int	cancelled;		/* TRUE if dialog was cancelled */
	short	type;			/* CPU type */
	long	p;			/* CPU speed */
	short	hours;
} speed_packet;

typedef struct rp {
	int	cancelled;		/* TRUE if dialog was cancelled */
	long	start;			/* Range choosen */
	long	end;			/* Range choosen */
	long	fudge;			/* Max end value fudge factor */
} range_packet;


/****************************************************************************

    FUNCTION: MainWndProc (HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages

    MESSAGES:

	WM_COMMAND    - application menu (About dialog box)
	WM_DESTROY    - destroy window

    COMMENTS:

	To process the IDM_ABOUT message, call MakeProcInstance() to get the
	current instance address of the About() function.  Then call Dialog
	box which will create the box according to the information in your
	prime.rc file and turn control over to the About() function.	When
	it returns, free the instance address.

****************************************************************************/

long FAR PASCAL MainWndProc (
	HWND	hWnd,
	UINT	message,
	WORD	wParam,
	LONG	lParam)
{
	switch (message) {
	case WM_COMMAND:	   /* message: command from app menu */

	switch (wParam) {

	case IDM_RANGE:
		{
		FARPROC lpProcRange;
		range_packet pkt;

		lpProcRange = MakeProcInstance (Range, hInst);

		DialogBoxParam (hInst,		/* current instance */
				"RangeBox",	/* resource to use */
				hWnd,		/* parent handle */
				lpProcRange,	/* Start() instance address*/
				(LPARAM) &pkt);

		FreeProcInstance (lpProcRange);

		if (!pkt.cancelled) {
			getWorkFromDatabase (pkt.start, pkt.end, FALSE, FALSE);
			startRange ();
		}
		break;
		}

	case IDM_RANGEFACTOR:
		{
		FARPROC lpProcRange;
		range_packet pkt;

		lpProcRange = MakeProcInstance (Range, hInst);

		DialogBoxParam (hInst,		/* current instance */
				"RangeFactorBox",/* resource to use */
				hWnd,		/* parent handle */
				lpProcRange,	/* Start() instance address*/
				(LPARAM) &pkt);

		FreeProcInstance (lpProcRange);

		if (!pkt.cancelled) {
			getWorkFromDatabase (pkt.start, pkt.end, TRUE, FALSE);
			startRange ();
		}
		break;
		}

	case IDM_RANGEDBLCHK:
		{
		FARPROC lpProcRange;
		range_packet pkt;

		lpProcRange = MakeProcInstance (Range, hInst);

		DialogBoxParam (hInst,		/* current instance */
				"RangeDblChkBox",/* resource to use */
				hWnd,		/* parent handle */
				lpProcRange,	/* Start() instance address*/
				(LPARAM) &pkt);

		FreeProcInstance (lpProcRange);

		if (!pkt.cancelled) {
			getWorkFromDatabase (pkt.start, pkt.end, FALSE, TRUE);
			startRange ();
		}
		break;
		}

	case IDM_CONTINUE:
		if (init32 ()) {
			primeContinue ();
			term32 ();
		}
		break;

	case IDM_RANGESTATUS:
		rangeStatus ();
		break;

	case IDM_EXIT:
		goto done;

// Advanced menu

	case IDM_PASSWORD:
		{
		FARPROC lpProcStart;

		lpProcStart = MakeProcInstance (Password, hInst);
		DialogBoxParam (hInst,		/* current instance */
				"PasswordBox",	/* resource to use */
				hWnd,		/* parent handle */
				lpProcStart,	/* Start() instance address*/
				(LPARAM) NULL);
		FreeProcInstance (lpProcStart);
		enable_advanced (hWnd);
		break;
		}

	case IDM_START:
		{
		FARPROC lpProcStart;
		start_packet pkt;

		lpProcStart = MakeProcInstance (Start, hInst);

		DialogBoxParam (hInst,		/* current instance */
				"StartBox",	/* resource to use */
				hWnd,		/* parent handle */
				lpProcStart,	/* Start() instance address*/
				(LPARAM) &pkt);

		FreeProcInstance (lpProcStart);

		if (!pkt.cancelled && init32 ()) {
			IniFileOpen (WORKTODO_FILE, 1);
			IniInsertLineAsInt (WORKTODO_FILE, 1, "AdvancedTest", pkt.p);
			primeContinue ();
			term32 ();
		}
		break;
		}

	case IDM_TIME:
		{
		FARPROC lpProcStart;
		time_packet pkt;

		lpProcStart = MakeProcInstance (Time, hInst);

		DialogBoxParam (hInst,		/* current instance */
				"TimeBox",	/* resource to use */
				hWnd,		/* parent handle */
				lpProcStart,	/* Start() instance address*/
				(LPARAM) &pkt);

		FreeProcInstance (lpProcStart);

		if (!pkt.cancelled && init32 ()) {
			primeTime (pkt.p, pkt.iterations);
			term32 ();
		}

		break;
		}

	case IDM_FACTOR:
		{
		FARPROC lpProcFactor;
		factor_packet pkt;

		lpProcFactor = MakeProcInstance (Factor, hInst);

		DialogBoxParam (hInst,		/* current instance */
				"FactorBox",	/* resource to use */
				hWnd,		/* parent handle */
				lpProcFactor,
				(LPARAM) &pkt);

		FreeProcInstance (lpProcFactor);

		if (!pkt.cancelled && init32 ()) {
			primeSieve (pkt.p1, pkt.p2,
				    pkt.b1, pkt.b2, pkt.ff, 0);
			term32 ();
		}
		break;
		}

// Options menu

	case IDM_SPEED:
		{
		FARPROC lpProcSpeed;
		speed_packet pkt;

		lpProcSpeed = MakeProcInstance (Speed, hInst);

		DialogBoxParam (hInst,		/* current instance */
				"CpuSpeedBox",	/* resource to use */
				hWnd,		/* parent handle */
				lpProcSpeed,	/* Debug() instance address*/
				(LPARAM) &pkt);

		FreeProcInstance (lpProcSpeed);

		if (!pkt.cancelled) {
			CPU_TYPE = pkt.type;
			CPU_SPEED = pkt.p;
			CPU_HOURS = pkt.hours;
			if (CPU_SPEED != (unsigned long) pkt.p ||
			    CPU_TYPE != (unsigned int) pkt.type ||
			    CPU_HOURS != (unsigned int) pkt.hours) {
				ROLLING_AVERAGE = 1000;
				IniWriteInt (LOCALINI_FILE, "RollingAverage", 1000);
				IniWriteInt (LOCALINI_FILE, "RollingStartTime", 0);
				UpdateEndDates ();
			}
			CPU_TYPE = pkt.type;
			CPU_SPEED = pkt.p;
			CPU_HOURS = pkt.hours;
			IniWriteInt (LOCALINI_FILE, "CPUType", CPU_TYPE);
			IniWriteInt (LOCALINI_FILE, "CPUSpeed", CPU_SPEED);
			IniWriteInt (LOCALINI_FILE, "CPUHours", CPU_HOURS);
		}

		break;
		}

	case IDM_OUTPUT:
		{
		FARPROC lpProcOutput;

		lpProcOutput = MakeProcInstance (Pref, hInst);
		DialogBoxParam (hInst,		/* current instance */
				"OutputBox",	/* resource to use */
				hWnd,		/* parent handle */
				lpProcOutput,
				(LPARAM) NULL);

		FreeProcInstance (lpProcOutput);
		break;
		}

	case IDM_SELFTEST:
		if (init32 ()) {
			selfTest (0);
			term32 ();
		}
		break;

	case IDM_TORTURETEST:
		if (init32 ()) {
			selfTest (1);
			term32 ();
		}
		break;

	case IDM_ERRCHK:
		ERRCHK = !ERRCHK;
		IniWriteInt (INI_FILE, "ErrorCheck", ERRCHK);
		CheckMenuItem (GetSubMenu (GetMenu (hWnd), 1),
				IDM_ERRCHK,
				ERRCHK ? MF_CHECKED : MF_UNCHECKED);
		break;

// Help menu

	case IDM_ABOUT:
		{
		FARPROC lpProcAbout;

		lpProcAbout = MakeProcInstance (About, hInst);

		DialogBox (hInst,	/* current instance */
			   "AboutBox",	/* resource to use */
			   hWnd,	/* parent handle */
			   lpProcAbout);/* About() instance address*/

		FreeProcInstance (lpProcAbout);
		break;
		}

	/* Lets Windows process message */

	default:
		return (DefWindowProc (hWnd, message, wParam, lParam));
	}
	break;

	case WM_SIZE:
		InvalidateRect (hWnd, NULL, TRUE);
		UpdateWindow (hWnd);
		return 0;

	case WM_PAINT: {
		PAINTSTRUCT ps;
		RECT	r;
		int	y;
		int	i;
		HDC hdc = BeginPaint(hWnd, &ps);
		GetClientRect (hWnd, &r);
		if (lines[0] != NULL)
		for (y = r.bottom, i = 1; y > 0 && i < NumLines; i++) {
			y -= charHeight;
			TextOut (ps.hdc, 0, y, lines[i], strlen (lines[i]));
		}
		EndPaint(hWnd, &ps);
		return 0;
		}
        
	case WM_DESTROY:		  /* message: window being destroyed*/
done:		if (HLIB) FreeLibrary (HLIB);
		PostQuitMessage (0);
		break;

	default:			  /* Passes it on if unproccessed */
		return (DefWindowProc (hWnd, message, wParam, lParam));
	}
	return (NULL);
}


/****************************************************************************

    FUNCTION: Pref (HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "Clear" dialog box

    MESSAGES:

	WM_INITDIALOG - initialize dialog box
	WM_COMMAND    - Input received

    COMMENTS:

	No initialization is needed for this particular dialog box, but TRUE
	must be returned to Windows.

	Wait for user to click on "Ok" button, then close the dialog box.

****************************************************************************/

BOOL FAR PASCAL Pref (hDlg, message, wParam, lParam)
	HWND	hDlg;			/* window handle of the dialog box */
	unsigned message;		/* type of message */
	WORD	wParam;			/* message-specific information */
	LONG	lParam;
{
	char	buf[20];

	switch (message) {
	case WM_INITDIALOG:	   /* message: initialize dialog box */
		sprintf (buf, "%d", ITER_OUTPUT);
		SetDlgItemText (hDlg, ID_P1, buf);
		sprintf (buf, "%ld", DISK_WRITE_TIME);
		SetDlgItemText (hDlg, ID_P3, buf);
		return (TRUE);

	case WM_COMMAND:		    /* message: received a command */
		if (wParam == IDOK) {            /* "OK" box selected? */
			GetDlgItemText (hDlg, ID_P1, buf, sizeof (buf));
			ITER_OUTPUT = atoi (buf);
			if (ITER_OUTPUT < 1) ITER_OUTPUT = 1;
			IniWriteInt (INI_FILE, "OutputIterations", ITER_OUTPUT);
			GetDlgItemText (hDlg, ID_P3, buf, sizeof (buf));
			DISK_WRITE_TIME = atoi (buf);
			IniWriteInt (INI_FILE, "DiskWriteTime", DISK_WRITE_TIME);
			EndDialog (hDlg, TRUE);	    /* Exits the dialog box */
			return (TRUE);
		}
		if (wParam == IDCANCEL) {
			EndDialog (hDlg, TRUE);	    /* Exits the dialog box */
			return (TRUE);
		}
	}
	return (FALSE);			    /* Didn't process a message */
}

/****************************************************************************

    FUNCTION: Factor (HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "Start" dialog box

    MESSAGES:

	WM_INITDIALOG - initialize dialog box
	WM_COMMAND    - Input received

    COMMENTS:

	No initialization is needed for this particular dialog box, but TRUE
	must be returned to Windows.

	Wait for user to click on "Ok" button, then close the dialog box.

****************************************************************************/

char NEAR BITSERR[] = "This program can only test factors up to 62 bits.";
char NEAR SMALLERR[] = "This number is too small to test with this program.";
char NEAR LARGEERR[] = "This number is too large to test with this program.";
char NEAR NOTPRIMEERR[] = "This number is not prime, there is no need to test it.";
char NEAR SPEEDERR[] = "The CPU speed must be at least 25 MHz.";

BOOL FAR PASCAL Factor (hDlg, message, wParam, lParam)
	HWND	hDlg;			/* window handle of the dialog box */
	unsigned message;		/* type of message */
	WORD	wParam;			/* message-specific information */
	LONG	lParam;
{
static	factor_packet FAR * NEAR pkt;

	switch (message) {
	case WM_INITDIALOG:	   /* message: initialize dialog box */
		pkt = (factor_packet FAR *) lParam;
		SetDlgItemText (hDlg, ID_B1, "1");
		SetDlgItemText (hDlg, ID_B2, "48");
		return (TRUE);

	case WM_COMMAND:		    /* message: received a command */
		if (wParam == IDOK) {            /* "OK" box selected? */
			char	buf[20];

			GetDlgItemText (hDlg, ID_P1, buf, sizeof (buf));
			pkt->p1 = atol (buf);
			GetDlgItemText (hDlg, ID_P2, buf, sizeof (buf));
			pkt->p2 = atol (buf);
			if (pkt->p1 < 1000 || pkt->p2 < 1000) {
				MessageBox (hDlg, SMALLERR, CAPTION,
					    MB_ICONHAND | MB_OK);
				return (TRUE);
			}
			GetDlgItemText (hDlg, ID_B1, buf, sizeof (buf));
			pkt->b1 = atoi (buf);
			GetDlgItemText (hDlg, ID_B2, buf, sizeof (buf));
			pkt->b2 = atoi (buf);
			if (pkt->b1 > 62 || pkt->b2 > 62) {
				MessageBox (hDlg, BITSERR, CAPTION,
					    MB_ICONHAND | MB_OK);
				return (TRUE);
			}
 			pkt->ff = IsDlgButtonChecked (hDlg, ID_FF);
			pkt->cancelled = FALSE;
			EndDialog (hDlg, TRUE);	    /* Exits the dialog box */
			return (TRUE);
		}
		if (wParam == IDCANCEL) {
			pkt->cancelled = TRUE;
			EndDialog (hDlg, TRUE);	    /* Exits the dialog box */
			return (TRUE);
		}
	}
	return (FALSE);			    /* Didn't process a message */
}

/****************************************************************************

    FUNCTION: Start (HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "Start" dialog box

    MESSAGES:

	WM_INITDIALOG - initialize dialog box
	WM_COMMAND    - Input received

    COMMENTS:

	No initialization is needed for this particular dialog box, but TRUE
	must be returned to Windows.

	Wait for user to click on "Ok" button, then close the dialog box.

****************************************************************************/

BOOL FAR PASCAL Start (hDlg, message, wParam, lParam)
	HWND	hDlg;			/* window handle of the dialog box */
	unsigned message;		/* type of message */
	WORD	wParam;			/* message-specific information */
	LONG	lParam;
{
static	start_packet FAR * NEAR pkt;

	switch (message) {
	case WM_INITDIALOG:	   /* message: initialize dialog box */
		pkt = (start_packet FAR *) lParam;
		return (TRUE);

	case WM_COMMAND:		    /* message: received a command */
		if (wParam == IDOK) {            /* "OK" box selected? */
			char	buf[20];
			long	i;

			GetDlgItemText (hDlg, ID_P, buf, sizeof (buf));
			pkt->cancelled = FALSE;
			pkt->p = atol (buf);
			if (pkt->p < MIN_PRIME) {
				MessageBox (hDlg, SMALLERR, CAPTION,
					    MB_ICONHAND | MB_OK);
				return (TRUE);
			}
			if (pkt->p > MAX_PRIME) {
				MessageBox (hDlg, LARGEERR, CAPTION,
					    MB_ICONHAND | MB_OK);
				return (TRUE);
			}
			for (i = 2; i * i <= pkt->p; i++) {
				if (pkt->p % i == 0) {
					MessageBox (hDlg, NOTPRIMEERR, CAPTION,
						    MB_ICONHAND | MB_OK);
					return (TRUE);
				}
			}
			EndDialog (hDlg, TRUE);	    /* Exits the dialog box */
			return (TRUE);
		}
		if (wParam == IDCANCEL) {
			pkt->cancelled = TRUE;
			EndDialog (hDlg, TRUE);	    /* Exits the dialog box */
			return (TRUE);
		}
	}
	return (FALSE);			    /* Didn't process a message */
}

/****************************************************************************

    FUNCTION: Range (HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "Range" dialog box

    MESSAGES:

	WM_INITDIALOG - initialize dialog box
	WM_COMMAND    - Input received

    COMMENTS:

	No initialization is needed for this particular dialog box, but TRUE
	must be returned to Windows.

	Wait for user to click on "Ok" button, then close the dialog box.

****************************************************************************/

BOOL FAR PASCAL Range (hDlg, message, wParam, lParam)
	HWND	hDlg;			/* window handle of the dialog box */
	unsigned message;		/* type of message */
	WORD	wParam;			/* message-specific information */
	LONG	lParam;
{
static	range_packet FAR * NEAR pkt;

	switch (message) {
	case WM_INITDIALOG:	   /* message: initialize dialog box */
		pkt = (range_packet FAR *) lParam;
		SetDlgItemText (hDlg, ID_P, "0");
		SetDlgItemText (hDlg, ID_P1, "0");
		return (TRUE);

	case WM_COMMAND:		    /* message: received a command */
		if (wParam == IDOK) {            /* "OK" box selected? */
			char	buf[20];

			GetDlgItemText (hDlg, ID_P, buf, sizeof (buf));
			pkt->start = atol (buf);
			GetDlgItemText (hDlg, ID_P1, buf, sizeof (buf));
			pkt->end = atol (buf);
			if (checkRange(hDlg, pkt->start, pkt->end, 0))
				return (TRUE);
			pkt->cancelled = FALSE;
			EndDialog (hDlg, TRUE);	    /* Exits the dialog box */
			return (TRUE);
		}
		if (wParam == IDCANCEL) {
			pkt->cancelled = TRUE;
			EndDialog (hDlg, TRUE);	    /* Exits the dialog box */
			return (TRUE);
		}
	}
	return (FALSE);			    /* Didn't process a message */
}

/****************************************************************************

    FUNCTION: Password (HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "Password" dialog box

    MESSAGES:

	WM_INITDIALOG - initialize dialog box
	WM_COMMAND    - Input received

    COMMENTS:

	No initialization is needed for this particular dialog box, but TRUE
	must be returned to Windows.

	Wait for user to click on "Ok" button, then close the dialog box.

****************************************************************************/

BOOL FAR PASCAL Password (hDlg, message, wParam, lParam)
	HWND	hDlg;			/* window handle of the dialog box */
	unsigned message;		/* type of message */
	WORD	wParam;			/* message-specific information */
	LONG	lParam;
{
	switch (message) {
	case WM_INITDIALOG:	   /* message: initialize dialog box */
		return (TRUE);

	case WM_COMMAND:		    /* message: received a command */
		if (wParam == IDOK) {            /* "OK" box selected? */
			char	buf[20];

			GetDlgItemText (hDlg, ID_P, buf, sizeof (buf));

			if (atol (buf) == 9876) {	// secret code
				ADVANCED_ENABLED = 1;
				IniWriteInt (INI_FILE, "Advanced", 1);
			}
			EndDialog (hDlg, TRUE);	    /* Exits the dialog box */
			return (TRUE);
		}
		if (wParam == IDCANCEL) {
			EndDialog (hDlg, TRUE);	    /* Exits the dialog box */
			return (TRUE);
		}
	}
	return (FALSE);			    /* Didn't process a message */
}

/****************************************************************************

    FUNCTION: Time (HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "Time" dialog box

    MESSAGES:

	WM_INITDIALOG - initialize dialog box
	WM_COMMAND    - Input received

    COMMENTS:

	No initialization is needed for this particular dialog box, but TRUE
	must be returned to Windows.

	Wait for user to click on "Ok" button, then close the dialog box.

****************************************************************************/

BOOL FAR PASCAL Time (hDlg, message, wParam, lParam)
	HWND	hDlg;			/* window handle of the dialog box */
	unsigned message;		/* type of message */
	WORD	wParam;			/* message-specific information */
	LONG	lParam;
{
static	time_packet FAR * NEAR pkt;

	switch (message) {
	case WM_INITDIALOG:	   /* message: initialize dialog box */
		pkt = (time_packet FAR *) lParam;
		SetDlgItemText (hDlg, ID_P, "800000");
		SetDlgItemText (hDlg, ID_ITER, "10");
		return (TRUE);

	case WM_COMMAND:		    /* message: received a command */
		if (wParam == IDOK) {            /* "OK" box selected? */
			char	buf[20];

			pkt->cancelled = FALSE;
			GetDlgItemText (hDlg, ID_P, buf, sizeof (buf));
			pkt->p = atol (buf);
			GetDlgItemText (hDlg, ID_ITER, buf, sizeof (buf));
			pkt->iterations = atol (buf);
			if (pkt->p < MIN_PRIME) {
				MessageBox (hDlg, SMALLERR, CAPTION,
					    MB_ICONHAND | MB_OK);
				return (TRUE);
			}
			if (pkt->p > MAX_PRIME) {
				MessageBox (hDlg, LARGEERR, CAPTION,
					    MB_ICONHAND | MB_OK);
				return (TRUE);
			}
			EndDialog (hDlg, TRUE);	    /* Exits the dialog box */
			return (TRUE);
		}
		if (wParam == IDCANCEL) {
			pkt->cancelled = TRUE;
			EndDialog (hDlg, TRUE);	    /* Exits the dialog box */
			return (TRUE);
		}
	}
	return (FALSE);			    /* Didn't process a message */
}

/****************************************************************************

    FUNCTION: Speed (HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "CPU speed" dialog box

    MESSAGES:

	WM_INITDIALOG - initialize dialog box
	WM_COMMAND    - Input received

    COMMENTS:

	No initialization is needed for this particular dialog box, but TRUE
	must be returned to Windows.

	Wait for user to click on "Ok" button, then close the dialog box.

****************************************************************************/

BOOL FAR PASCAL Speed (hDlg, message, wParam, lParam)
	HWND	hDlg;			/* window handle of the dialog box */
	unsigned message;		/* type of message */
	WORD	wParam;			/* message-specific information */
	LONG	lParam;
{
static	speed_packet FAR * NEAR pkt;
	char	buf[10];

	switch (message) {
	case WM_INITDIALOG:	   /* message: initialize dialog box */
		pkt = (speed_packet FAR *) lParam;
		CheckRadioButton (hDlg, IDC_CYRIX, IDC_PRO,
			CPU_TYPE == 3 ? IDC_CYRIX :
			CPU_TYPE == 4 ? IDC_486 :
			CPU_TYPE == 5 ? IDC_PENTIUM : IDC_PRO);
		sprintf (buf, "%ld", CPU_SPEED);
		SetDlgItemText (hDlg, ID_P, buf);
		sprintf (buf, "%d", CPU_HOURS);
		SetDlgItemText (hDlg, ID_P2, buf);
		return (TRUE);

	case WM_COMMAND:		    /* message: received a command */
		if (wParam == IDOK) {            /* "OK" box selected? */
			GetDlgItemText (hDlg, ID_P, buf, sizeof (buf));
			pkt->p = atol (buf);
			if (pkt->p < 25) {
				MessageBox (hDlg, SPEEDERR, CAPTION,
					    MB_ICONHAND | MB_OK);
				return (TRUE);
			}
			GetDlgItemText (hDlg, ID_P2, buf, sizeof (buf));
			pkt->hours = (unsigned int) atol (buf);
			if (pkt->hours > 24) pkt->hours = 24;
			if (pkt->hours < 1) pkt->hours = 1;
			pkt->type = 4;
			if (IsDlgButtonChecked(hDlg,IDC_CYRIX)) pkt->type = 3;
			if (IsDlgButtonChecked(hDlg,IDC_486)) pkt->type = 4;
			if (IsDlgButtonChecked(hDlg,IDC_PENTIUM)) pkt->type=5;
			if (IsDlgButtonChecked(hDlg,IDC_PRO)) pkt->type = 6;
			pkt->cancelled = FALSE;
			EndDialog (hDlg, TRUE);	    /* Exits the dialog box */
			return (TRUE);
		}
		if (wParam == IDCANCEL) {
			pkt->cancelled = TRUE;
			EndDialog (hDlg, TRUE);	    /* Exits the dialog box */
			return (TRUE);
		}
	}
	return (FALSE);			    /* Didn't process a message */
}

/****************************************************************************

    FUNCTION: About (HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "About" dialog box

    MESSAGES:

	WM_INITDIALOG - initialize dialog box
	WM_COMMAND    - Input received

    COMMENTS:

	No initialization is needed for this particular dialog box, but TRUE
	must be returned to Windows.

	Wait for user to click on "Ok" button, then close the dialog box.

****************************************************************************/

BOOL FAR PASCAL About (hDlg, message, wParam, lParam)
	HWND	hDlg;			/* window handle of the dialog box */
	unsigned message;		/* type of message */
	WORD	wParam;			/* message-specific information */
	LONG	lParam;
{
	switch (message) {
	case WM_INITDIALOG:	   /* message: initialize dialog box */
		return (TRUE);

	case WM_COMMAND:		    /* message: received a command */
		if (wParam == IDOK ||		/* "OK" box selected? */
                    wParam == IDCANCEL) {	/* System menu close cmd? */
			EndDialog (hDlg, TRUE);	/* Exits the dialog box */
			return (TRUE);
		}
		break;
	}
	return (FALSE);			    /* Didn't process a message */
}

//  Abort dialog procedure
BOOL FAR PASCAL AbortDlgProc(
	HWND hDlg,
	UINT msg,
	UINT wPAram,
	LONG lParam)
{
	switch (msg) {
		case WM_INITDIALOG:
			SetWindowText(hDlg, "Prime");
			EnableMenuItem(GetSystemMenu(hDlg, FALSE), SC_CLOSE, MF_GRAYED);
			return TRUE;
			
		case WM_COMMAND:
			bAbort = TRUE;
			return TRUE;
	}
	return FALSE;
}

int escapeCheck ()
{
	MSG tm;

	while (!bAbort && PeekMessage(&tm, NULL, 0, 0, PM_REMOVE)) {
		if (!hDlgAbort || !IsDialogMessage(hDlgAbort, &tm)) {
			TranslateMessage(&tm);
			DispatchMessage(&tm);
		}
	}
	return bAbort;
}

/*************************************************************************/
/*			Output Text to Main Window			 */
/*************************************************************************/

void FAR PASCAL InternalLineFeed ()
{
	char	*p;

	if (lines[0] != NULL) {
		p = lines[NumLines-1];
		memmove (&lines[1], &lines[0], (NumLines-1)*sizeof(char *));
		lines[0] = p;
		*p = 0;
	}

	if (charHeight == 0)
		charHeight = HIWORD (GetTextExtent (GetDC (hMainWnd), "A", 1));
	ScrollWindow (hMainWnd, 0, -charHeight, NULL, NULL);
	UpdateWindow (hMainWnd);
}

void OutputStr (
	LPSTR	str)
{
	char	*p;
	int	i;

	if (lines[0] == NULL) {
		for (i = 0; i < NumLines; i++) {
			lines[i] = &linebuf[i*LineSize];
			*lines[i] = 0;
		}
	}
	p = lines[0] + strlen (lines[0]);
	for ( ; *str; str++) {
		if (*str == '\n') *p = 0, InternalLineFeed (), p = lines[0];
		else *p++ = *str;
	}
	*p = 0;
}
