; Copyright 1995-1998 Just For Fun Software, Inc., all rights reserved
; Author:  George Woltman
; Email: woltman@magicnet.net
;
; This routine implements a discrete weighted transform to quickly multiply
; two numbers.  This code derived from the Lucas-Lehmer assembly code, but
; only handles small numbers (one pass FFTs) for now.
;
; This code handles FFT sizes from 32 doubles up to 1024 doubles.
;
; You will not stand a chance of understanding any of this code without
; thoroughly familiarizing yourself with fast fourier transforms.  This
; code was adapted from an algorithm described in Richard Crandall's article
; on Discrete Weighted Transforms and Large-Integer Arithmetic.
;

	TITLE   setup

	.386

_TEXT32 SEGMENT PARA USE32 PUBLIC 'DATA'

EXTRN	_ERRCHK:DWORD
EXTRN	_PARG:DWORD
EXTRN	_FFTLEN:DWORD
EXTRN	_SRCARG:DWORD
EXTRN	_SRC2ARG:DWORD
EXTRN	_DESTARG:DWORD
EXTRN	_NORM_ARRAY:DWORD
EXTRN	_PLUS1:DWORD

PUBLIC	PLUS1_SETUP_ADDR
EXTRN	PMAX:DWORD
EXTRN	FFT_1_ADDR:DWORD
EXTRN	FFT_2_ADDR:DWORD
EXTRN	FFT_3_ADDR:DWORD
EXTRN	FFT_4_ADDR:DWORD
EXTRN	ECHK_ADDR:DWORD
EXTRN	NOECHK_ADDR:DWORD

;
; Global variables needed in squaring
;

	align	32
PLUS1_SETUP_ADDR DD	OFFSET PLUS1_SETUP ; Addr of PLUS1_SETUP
		; These values only used during setup
jmptable	DD	755,	32,	OFFSET fft32_1, OFFSET fft32_2
		DD			OFFSET fft32_3, OFFSET fft32_4, 0
		DD	1497,	64,	OFFSET fft64_1, OFFSET fft64_2
		DD			OFFSET fft64_3, OFFSET fft64_4, 0
		DD	2951,	128,	OFFSET fft128_1, OFFSET fft128_2
		DD			OFFSET fft128_3, OFFSET fft128_4, 0
		DD	5823,	256,	OFFSET fft256_1, OFFSET fft256_2
		DD			OFFSET fft256_3, OFFSET fft256_4, 0
		DD	11571,	512,	OFFSET fft512_1, OFFSET fft512_2
		DD			OFFSET fft512_3, OFFSET fft512_4, 0
		DD	22831,	1024,	OFFSET fft1024_1, OFFSET fft1024_2
		DD			OFFSET fft1024_3, OFFSET fft1024_4, 0
_TEXT32	ENDS

	ASSUME  CS: _TEXT32, DS: _TEXT32, SS: _TEXT32, ES: _TEXT32

assign_addrs	MACRO	sc0_size, sc1_size
norm_mults	EQU	20010000h
norm_aux1	EQU	norm_mults + 1024 * 24
sine_cosine0	EQU	norm_aux1 + 64
sine_cosine1	EQU	sine_cosine0 + sc0_size
plus1_mults	EQU	sine_cosine1 + sc1_size
sc1_gap		EQU	32
		ENDM

sc1_distances MACRO
	GAP = sc1_gap
	SCF = 0
	SCD = 48
	ENDM

INCLUDE	unravel.mac
INCLUDE	lucas1.mac
INCLUDE plus1.mac

	assign_addrs 0, 8192
	dist1 =	8

_TEXT32	SEGMENT

; Setup routine

	PUBLIC	PLUS1_SETUP
PLUS1_SETUP PROC NEAR

; Save p, compute fftlen

	mov	eax, _PARG		; load p
	mov	esi, OFFSET jmptable	; table to compute FFTLEN
flp:	mov	edx, [esi]		; load maximum p
	cmp	eax, edx		; is our p less than maximum p?
	jle	short fdn		; yes, we've found our fftlen
flp1:	cmp	DWORD PTR [esi], 0	; look for zero terminator
	lea	esi, [esi+4]		; next table entry
	jnz	short flp1
	jmp	short flp		; loop
fdn:	mov	pmax, edx
	mov	ecx, [esi+4]		; Save FFTLEN
	mov	_FFTLEN, ecx
	mov	ecx, [esi+8]		; Save 4 fft routine offsets
	mov	fft_1_addr, ecx
	mov	ecx, [esi+12]
	mov	fft_2_addr, ecx
	mov	ecx, [esi+16]
	mov	fft_3_addr, ecx
	mov	ecx, [esi+20]
	mov	fft_4_addr, ecx

; Compute the array of sine and cosine/sine values

	sine_cosine_setup

; Create multipliers for 2^p+1 arithmetic

	plus1_mult_setup

; Return

	ret
plus1_setup ENDP


;; All the FFT routines for each FFT length

_gw_ffts PROC NEAR
fft32_2:	type2	32
fft32_3:	type3	32
fft32_1:	fft32	1
fft32_4:	fft32	4
fft64_2:	type2	64
fft64_3:	type3	64
fft64_1:	fft64	1
fft64_4:	fft64	4
fft128_2:	type2	128
fft128_3:	type3	128
fft128_1:	fft128	1
fft128_4:	fft128	4
fft256_2:	type2	256
fft256_3:	type3	256
fft256_1:	fft256	1
fft256_4:	fft256	4
fft512_2:	type2	512
fft512_3:	type3	512
fft512_1:	fft512	1
fft512_4:	fft512	4
fft1024_2:	type2	1024
fft1024_3:	type3	1024
fft1024_1:	fft1024	1
fft1024_4:	fft1024	4

_gw_ffts ENDP

_TEXT32	ENDS
END
